@@ -47,27 +47,32 @@
 #include <utility>
 #include <vector>
 
-#include "boost/algorithm/string.hpp"
-#include "boost/bind.hpp"
-#include "boost/filesystem.hpp"
-#include "boost/foreach.hpp"
+#include "ament_index_cpp/get_package_prefix.hpp"
+#include "ament_index_cpp/get_package_share_directory.hpp"
+#include "ament_index_cpp/get_resource.hpp"
+#include "ament_index_cpp/get_resources.hpp"
 #include "class_loader/class_loader.hpp"
-
-#include "ros/package.h"
+#include "rcpputils/filesystem_helper.hpp"
+#include "rcpputils/shared_library.hpp"
+#include "rcutils/logging_macros.h"
 
 #include "./class_loader.hpp"
+#include "./impl/split.hpp"
 
 #ifdef _WIN32
-const std::string os_pathsep(";");  // NOLINT
+#define CLASS_LOADER_IMPL_OS_PATHSEP ";"
 #else
-const std::string os_pathsep(":");  // NOLINT
+#define CLASS_LOADER_IMPL_OS_PATHSEP ":"
 #endif
 
 namespace pluginlib
 {
+
 template<class T>
 ClassLoader<T>::ClassLoader(
-  std::string package, std::string base_class, std::string attrib_name,
+  std::string package,
+  std::string base_class,
+  std::string attrib_name,
   std::vector<std::string> plugin_xml_paths)
 : plugin_xml_paths_(plugin_xml_paths),
   package_(package),
@@ -80,27 +85,31 @@ ClassLoader<T>::ClassLoader(
   lowlevel_class_loader_(false)
   /***************************************************************************/
 {
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Creating ClassLoader, base = %s, address = %p",
-    base_class.c_str(), this);
-  if (ros::package::getPath(package_).empty()) {
-    throw pluginlib::ClassLoaderException("Unable to find package: " + package_);
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader", "Creating ClassLoader, base = %s, address = %p",
+    base_class.c_str(), static_cast<void *>(this));
+  try {
+    ament_index_cpp::get_package_prefix(package_);
+  } catch (const ament_index_cpp::PackageNotFoundError & exception) {
+    // rethrow as class loader exception, package name is in the error message already.
+    throw pluginlib::ClassLoaderException(exception.what());
   }
 
   if (0 == plugin_xml_paths_.size()) {
     plugin_xml_paths_ = getPluginXmlPaths(package_, attrib_name_);
   }
   classes_available_ = determineAvailableClasses(plugin_xml_paths_);
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader",
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
     "Finished constructring ClassLoader, base = %s, address = %p",
-    base_class.c_str(), this);
+    base_class.c_str(), static_cast<void *>(this));
 }
 
 template<class T>
 ClassLoader<T>::~ClassLoader()
 /***************************************************************************/
 {
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Destroying ClassLoader, base = %s, address = %p",
-    getBaseClassType().c_str(), this);
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
+    "Destroying ClassLoader, base = %s, address = %p",
+    getBaseClassType().c_str(), static_cast<void *>(this));
 }
 
 
@@ -109,66 +118,43 @@ T * ClassLoader<T>::createClassInstance(const std::string & lookup_name, bool au
 /***************************************************************************/
 {
   // Note: This method is deprecated
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader",
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
     "In deprecated call createClassInstance(), lookup_name = %s, auto_load = %i.",
     (lookup_name.c_str()), auto_load);
 
   if (auto_load && !isClassLoaded(lookup_name)) {
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader",
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
       "Autoloading class library before attempting to create instance.");
     loadLibraryForClass(lookup_name);
   }
 
   try {
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader",
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
       "Attempting to create instance through low-level MultiLibraryClassLoader...");
     T * obj = lowlevel_class_loader_.createUnmanagedInstance<T>(getClassType(lookup_name));
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Instance created with object pointer = %p", obj);
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
+      "Instance created with object pointer = %p", static_cast<void *>(obj));
 
     return obj;
   } catch (const class_loader::CreateClassException & ex) {
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "CreateClassException about to be raised for class %s",
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
+      "CreateClassException about to be raised for class %s",
       lookup_name.c_str());
     throw pluginlib::CreateClassException(ex.what());
   }
 }
 
 template<class T>
-boost::shared_ptr<T> ClassLoader<T>::createInstance(const std::string & lookup_name)
+std::shared_ptr<T> ClassLoader<T>::createSharedInstance(const std::string & lookup_name)
 /***************************************************************************/
 {
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Attempting to create managed instance for class %s.",
-    lookup_name.c_str());
-
-  if (!isClassLoaded(lookup_name)) {
-    loadLibraryForClass(lookup_name);
-  }
-
-  try {
-    std::string class_type = getClassType(lookup_name);
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "%s maps to real class type %s",
-      lookup_name.c_str(), class_type.c_str());
-
-    boost::shared_ptr<T> obj = lowlevel_class_loader_.createInstance<T>(class_type);
-
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "boost::shared_ptr to object of real type %s created.",
-      class_type.c_str());
-
-    return obj;
-  } catch (const class_loader::CreateClassException & ex) {
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader",
-      "Exception raised by low-level multi-library class loader when attempting "
-      "to create instance of class %s.",
-      lookup_name.c_str());
-    throw pluginlib::CreateClassException(ex.what());
-  }
+  return createUniqueInstance(lookup_name);
 }
 
-#if __cplusplus >= 201103L
 template<class T>
 UniquePtr<T> ClassLoader<T>::createUniqueInstance(const std::string & lookup_name)
 {
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader",
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
     "Attempting to create managed (unique) instance for class %s.",
     lookup_name.c_str());
 
@@ -178,30 +164,31 @@ UniquePtr<T> ClassLoader<T>::createUniqueInstance(const std::string & lookup_nam
 
   try {
     std::string class_type = getClassType(lookup_name);
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "%s maps to real class type %s",
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader", "%s maps to real class type %s",
       lookup_name.c_str(), class_type.c_str());
 
     UniquePtr<T> obj = lowlevel_class_loader_.createUniqueInstance<T>(class_type);
 
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "std::unique_ptr to object of real type %s created.",
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
+      "std::unique_ptr to object of real type %s created.",
       class_type.c_str());
 
     return obj;
   } catch (const class_loader::CreateClassException & ex) {
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader",
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
       "Exception raised by low-level multi-library class loader when attempting "
       "to create instance of class %s.",
       lookup_name.c_str());
     throw pluginlib::CreateClassException(ex.what());
   }
 }
-#endif
 
 template<class T>
 T * ClassLoader<T>::createUnmanagedInstance(const std::string & lookup_name)
 /***************************************************************************/
 {
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Attempting to create UNMANAGED instance for class %s.",
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
+    "Attempting to create UNMANAGED instance for class %s.",
     lookup_name.c_str());
 
   if (!isClassLoaded(lookup_name)) {
@@ -210,15 +197,17 @@ T * ClassLoader<T>::createUnmanagedInstance(const std::string & lookup_name)
 
   T * instance = 0;
   try {
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader",
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
       "Attempting to create instance through low level multi-library class loader.");
     std::string class_type = getClassType(lookup_name);
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "%s maps to real class type %s",
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader", "%s maps to real class type %s",
       lookup_name.c_str(), class_type.c_str());
     instance = lowlevel_class_loader_.createUnmanagedInstance<T>(class_type);
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Instance of type %s created.", class_type.c_str());
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
+      "Instance of type %s created.",
+      class_type.c_str());
   } catch (const class_loader::CreateClassException & ex) {
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader",
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
       "Exception raised by low-level multi-library class loader when attempting "
       "to create UNMANAGED instance of class %s.",
       lookup_name.c_str());
@@ -230,13 +219,45 @@ T * ClassLoader<T>::createUnmanagedInstance(const std::string & lookup_name)
 template<class T>
 std::vector<std::string> ClassLoader<T>::getPluginXmlPaths(
   const std::string & package,
-  const std::string & attrib_name,
-  bool force_recrawl)
+  const std::string & attrib_name)
 /***************************************************************************/
 {
   // Pull possible files from manifests of packages which depend on this package and export class
   std::vector<std::string> paths;
-  ros::package::getPlugins(package, attrib_name, paths, force_recrawl);
+  {
+    // the convention is to create an ament resource which a concatenation of
+    // the package name, "pluginlib", and the attribute being exported
+    // __ is used as the concatenation delimiter because it cannot be in a
+    // package name
+    std::string resource_name = package + "__pluginlib__" + attrib_name;
+    auto plugin_packages_with_prefixes = ament_index_cpp::get_resources(resource_name);
+    for (const auto & package_prefix_pair : plugin_packages_with_prefixes) {
+      // it is also convention to place the relative path to the plugin xml in
+      // the ament resource file
+      std::string resource_content;
+      {
+        using ament_index_cpp::get_resource;
+        if (!get_resource(resource_name, package_prefix_pair.first, resource_content)) {
+          RCUTILS_LOG_WARN_NAMED("pluginlib.ClassLoader",
+            "unexpectedly not able to find ament resource '%s' for package '%s'",
+            resource_name.c_str(),
+            package_prefix_pair.first.c_str()
+          );
+          continue;
+        }
+      }
+      // the content may contain multiple plugin description files
+      std::stringstream ss(resource_content);
+      std::string line;
+      while (std::getline(ss, line, '\n')) {
+        if (!line.empty()) {
+          // store the prefix for the package with a plugin and the relative path
+          // to the plugin xml file
+          paths.push_back(package_prefix_pair.second + "/" + line);
+        }
+      }
+    }
+  }
   return paths;
 }
 
@@ -250,7 +271,7 @@ std::map<std::string, ClassDesc> ClassLoader<T>::determineAvailableClasses(
   // seem to be correct.
   // With time I keep correcting small things, but a good rewrite is needed.
 
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Entering determineAvailableClasses()...");
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader", "Entering determineAvailableClasses()...");
   std::map<std::string, ClassDesc> classes_available;
 
   // Walk the list of all plugin XML files (variable "paths") that are exported by the build system
@@ -260,13 +281,13 @@ std::map<std::string, ClassDesc> ClassLoader<T>::determineAvailableClasses(
     try {
       processSingleXMLPluginFile(*it, classes_available);
     } catch (const pluginlib::InvalidXMLException & e) {
-      ROS_ERROR_NAMED("pluginlib.ClassLoader",
+      RCUTILS_LOG_ERROR_NAMED("pluginlib.ClassLoader",
         "Skipped loading plugin with error: %s.",
         e.what());
     }
   }
 
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Exiting determineAvailableClasses()...");
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader", "Exiting determineAvailableClasses()...");
   return classes_available;
 }
 
@@ -278,7 +299,7 @@ std::string ClassLoader<T>::extractPackageNameFromPackageXML(const std::string &
   document.LoadFile(package_xml_path.c_str());
   tinyxml2::XMLElement * doc_root_node = document.FirstChildElement("package");
   if (NULL == doc_root_node) {
-    ROS_ERROR_NAMED("pluginlib.ClassLoader",
+    RCUTILS_LOG_ERROR_NAMED("pluginlib.ClassLoader",
       "Could not find a root element for package manifest at %s.",
       package_xml_path.c_str());
     return "";
@@ -288,7 +309,7 @@ std::string ClassLoader<T>::extractPackageNameFromPackageXML(const std::string &
 
   tinyxml2::XMLElement * package_name_node = doc_root_node->FirstChildElement("name");
   if (NULL == package_name_node) {
-    ROS_ERROR_NAMED("pluginlib.ClassLoader",
+    RCUTILS_LOG_ERROR_NAMED("pluginlib.ClassLoader",
       "package.xml at %s does not have a <name> tag! Cannot determine package "
       "which exports plugin.",
       package_xml_path.c_str());
@@ -297,7 +318,7 @@ std::string ClassLoader<T>::extractPackageNameFromPackageXML(const std::string &
 
   const char* package_name_node_txt = package_name_node->GetText();
   if (NULL == package_name_node_txt) {
-    ROS_ERROR_NAMED("pluginlib.ClassLoader",
+    RCUTILS_LOG_ERROR_NAMED("pluginlib.ClassLoader",
       "package.xml at %s has an invalid <name> tag! Cannot determine package "
       "which exports plugin.",
       package_xml_path.c_str());
@@ -307,70 +328,103 @@ std::string ClassLoader<T>::extractPackageNameFromPackageXML(const std::string &
   return package_name_node_txt;
 }
 
-template<class T>
-std::vector<std::string> ClassLoader<T>::getCatkinLibraryPaths()
-/***************************************************************************/
-{
-  std::vector<std::string> lib_paths;
-  const char * env = std::getenv("CMAKE_PREFIX_PATH");
-  if (env) {
-    std::string env_catkin_prefix_paths(env);
-    std::vector<std::string> catkin_prefix_paths;
-    boost::split(catkin_prefix_paths, env_catkin_prefix_paths, boost::is_any_of(os_pathsep));
-    BOOST_FOREACH(std::string catkin_prefix_path, catkin_prefix_paths) {
-      boost::filesystem::path path(catkin_prefix_path);
-#if _WIN32
-      boost::filesystem::path bin("bin");
-      lib_paths.push_back((path / bin).string());
-#endif
-      boost::filesystem::path lib("lib");
-      lib_paths.push_back((path / lib).string());
-    }
-  }
-  return lib_paths;
-}
-
 template<class T>
 std::vector<std::string> ClassLoader<T>::getAllLibraryPathsToTry(
   const std::string & library_name,
   const std::string & exporting_package_name)
 /***************************************************************************/
 {
-  // Catkin-rosbuild Backwards Compatability Rules - Note library_name may be prefixed with
-  // relative path (e.g. "/lib/libFoo")
-  // 1. Try catkin library paths (catkin_find --libs) + library_name + extension
-  // 2. Try catkin library paths
-  //   (catkin_find -- libs) + stripAllButFileFromPath(library_name) + extension
-  // 3. Try export_pkg/library_name + extension
+  // To determine the common prefix of the paths to try, the prefix of the
+  // exporting package is retrieved.
+  // To that, various library folder names are added (lib, lib64, etc...)
+  // Additionally, "libexec" like folders are checked, using the package name
+  // as the libexec folder name within the library name.
+  // Finally the library name (just the file name, stripped of extra relative)
+  // with various extensions is concatenated to the various library directories.
+  //
+  // For example, if the package was 'rviz' and the library_name was
+  // 'librviz_default_plugins', these paths might be tried:
+  //
+  //   - <prefix for rviz>/lib/librviz_default_plugins.so
+  //   - <prefix for rviz>/lib64/librviz_default_plugins.so
+  //   - <prefix for rviz>/bin/rviz_default_plugins.dll
+  //   - <prefix for rviz>/lib/rviz/librviz_default_plugins.so
+  //   - <prefix for rviz>/lib64/rviz/librviz_default_plugins.so
+  //
+  // The extension, e.g. `.so`, might be different based on the operating
+  // system, e.g. it might be `.dylib` on macOS or `.dll` on Windows.
+  // Similarly, the library might have the `lib` prefix added or removed.
+  // Also, the library name might have a `d` added if the library is built
+  // debug, depending on the system.
 
-  std::vector<std::string> all_paths;
-  std::vector<std::string> all_paths_without_extension = getCatkinLibraryPaths();
-  all_paths_without_extension.push_back(getROSBuildLibraryPath(exporting_package_name));
-  bool debug_library_suffix = (0 == class_loader::systemLibrarySuffix().compare(0, 1, "d"));
-  std::string non_debug_suffix;
-  if (debug_library_suffix) {
-    non_debug_suffix = class_loader::systemLibrarySuffix().substr(1);
-  } else {
-    non_debug_suffix = class_loader::systemLibrarySuffix();
-  }
-  std::string library_name_with_extension = library_name + non_debug_suffix;
-  std::string stripped_library_name = stripAllButFileFromPath(library_name);
-  std::string stripped_library_name_with_extension = stripped_library_name + non_debug_suffix;
+  // TODO(wjwwood): probably should avoid "searching" and just embed the
+  // relative path to the libraries in the ament index, since CMake knows it
+  // at build time...
 
   const std::string path_separator = getPathSeparator();
 
-  for (unsigned int c = 0; c < all_paths_without_extension.size(); c++) {
-    std::string current_path = all_paths_without_extension.at(c);
-    all_paths.push_back(current_path + path_separator + library_name_with_extension);
-    all_paths.push_back(current_path + path_separator + stripped_library_name_with_extension);
-    // We're in debug mode, try debug libraries as well
-    if (debug_library_suffix) {
-      all_paths.push_back(
-        current_path + path_separator + library_name + class_loader::systemLibrarySuffix());
-      all_paths.push_back(
-        current_path + path_separator + stripped_library_name +
-        class_loader::systemLibrarySuffix());
+  std::vector<std::string> all_paths;  // result of all pairs to search
+
+  std::string package_prefix = ament_index_cpp::get_package_prefix(exporting_package_name);
+
+  // Setup the directories to look in.
+  std::vector<std::string> all_search_paths = {
+    // for now just try lib and lib64 (and their respective "libexec" directories)
+    package_prefix + path_separator + "lib",
+    package_prefix + path_separator + "lib64",
+    package_prefix + path_separator + "bin",  // also look in bin, for dll's on Windows
+    package_prefix + path_separator + "lib" + path_separator + exporting_package_name,
+    package_prefix + path_separator + "lib64" + path_separator + exporting_package_name,
+    package_prefix + path_separator + "bin" + path_separator + exporting_package_name,
+  };
+
+  std::string stripped_library_name = stripAllButFileFromPath(library_name);
+
+  std::string library_name_alternative;  // either lib<library> or <library> without lib prefix
+  const char * lib_prefix = "lib";
+  if (library_name.rfind(lib_prefix, 0) == 0) {
+    library_name_alternative = library_name.substr(strlen(lib_prefix));
+    RCUTILS_LOG_WARN_NAMED("pluginlib.ClassLoader",
+      "given plugin name '%s' should be '%s' for better portability",
+      library_name.c_str(),
+      library_name_alternative.c_str());
+  } else {
+    library_name_alternative = lib_prefix + library_name;
+  }
+  std::string stripped_library_name_alternative = stripAllButFileFromPath(library_name_alternative);
+
+  try {
+    // Setup the relative file paths to pair with the search directories above.
+    std::vector<std::string> all_relative_library_paths = {
+      rcpputils::get_platform_library_name(library_name),
+      rcpputils::get_platform_library_name(library_name_alternative),
+      rcpputils::get_platform_library_name(stripped_library_name),
+      rcpputils::get_platform_library_name(stripped_library_name_alternative)
+    };
+    std::vector<std::string> all_relative_debug_library_paths = {
+      rcpputils::get_platform_library_name(library_name, true),
+      rcpputils::get_platform_library_name(library_name_alternative, true),
+      rcpputils::get_platform_library_name(stripped_library_name, true),
+      rcpputils::get_platform_library_name(stripped_library_name_alternative, true)
+    };
+
+    for (auto && current_search_path : all_search_paths) {
+      for (auto && current_library_path : all_relative_library_paths) {
+        all_paths.push_back(current_search_path + path_separator + current_library_path);
+      }
+      for (auto && current_library_path : all_relative_debug_library_paths) {
+        all_paths.push_back(current_search_path + path_separator + current_library_path);
+      }
     }
+  } catch (const std::runtime_error & ex) {
+    throw std::runtime_error{ex.what()};
+  }
+
+  for (auto && path : all_paths) {
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
+      "[search path for '%s']: '%s'",
+      library_name.c_str(),
+      path.c_str());
   }
 
   return all_paths;
@@ -417,32 +471,35 @@ std::string ClassLoader<T>::getClassLibraryPath(const std::string & lookup_name)
 /***************************************************************************/
 {
   if (classes_available_.find(lookup_name) == classes_available_.end()) {
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Class %s has no mapping in classes_available_.",
-      lookup_name.c_str());
-    return "";
+    std::ostringstream error_msg;
+    error_msg << "Could not find library corresponding to plugin " << lookup_name <<
+      ". Make sure the plugin description XML file has the correct name of the library.";
+    throw pluginlib::LibraryLoadException(error_msg.str());
   }
   ClassMapIterator it = classes_available_.find(lookup_name);
   std::string library_name = it->second.library_name_;
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Class %s maps to library %s in classes_available_.",
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
+    "Class %s maps to library %s in classes_available_.",
     lookup_name.c_str(), library_name.c_str());
 
   std::vector<std::string> paths_to_try =
     getAllLibraryPathsToTry(library_name, it->second.package_);
 
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader",
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
     "Iterating through all possible paths where %s could be located...",
     library_name.c_str());
-  for (std::vector<std::string>::const_iterator it = paths_to_try.begin(); it != paths_to_try.end();
-    it++)
-  {
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Checking path %s ", it->c_str());
-    if (boost::filesystem::exists(*it)) {
-      ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Library %s found at explicit path %s.",
+  for (auto it = paths_to_try.begin(); it != paths_to_try.end(); it++) {
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader", "Checking path %s ", it->c_str());
+    if (rcpputils::fs::exists(*it)) {
+      RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader", "Library %s found at explicit path %s.",
         library_name.c_str(), it->c_str());
       return *it;
     }
   }
-  return "";
+  std::ostringstream error_msg;
+  error_msg << "Could not find library corresponding to plugin " << lookup_name <<
+    ". Make sure that the library '" << library_name << "' actually exists.";
+  throw pluginlib::LibraryLoadException(error_msg.str());
 }
 
 template<class T>
@@ -494,9 +551,8 @@ std::string ClassLoader<T>::getName(const std::string & lookup_name)
 /***************************************************************************/
 {
   // remove the package name to get the raw plugin name
-  std::vector<std::string> split;
-  boost::split(split, lookup_name, boost::is_any_of("/:"));
-  return split.back();
+  std::vector<std::string> result = pluginlib::impl::split(lookup_name, "/|:");
+  return result.back();
 }
 
 template<class T>
@@ -505,49 +561,32 @@ ClassLoader<T>::getPackageFromPluginXMLFilePath(const std::string & plugin_xml_f
 /***************************************************************************/
 {
   // Note: This method takes an input a path to a plugin xml file and must determine which
-  // package the XML file came from. This is not necessariliy the same thing as the member
+  // package the XML file came from. This is not necessarily the same thing as the member
   // variable "package_". The plugin xml file can be located anywhere in the source tree for a
   // package
 
-  // rosbuild:
-  // 1. Find nearest encasing manifest.xml
-  // 2. Once found, the name of the folder containg the manifest should be the
-  //   package name we are looking for
-  // 3. Confirm package is findable with rospack
-
-  // catkin:
+  // catkin and ament:
   // 1. Find nearest encasing package.xml
   // 2. Extract name of package from package.xml
 
   std::string package_name;
-  boost::filesystem::path p(plugin_xml_file_path);
-  boost::filesystem::path parent = p.parent_path();
+  rcpputils::fs::path p(plugin_xml_file_path);
+  rcpputils::fs::path parent = p.parent_path();
 
   // Figure out exactly which package the passed XML file is exported by.
   while (true) {
-    if (boost::filesystem::exists(parent / "package.xml")) {
-      std::string package_file_path = (boost::filesystem::path(parent / "package.xml")).string();
+    if (rcpputils::fs::exists(parent / "package.xml")) {
+      std::string package_file_path = (parent / "package.xml").string();
       return extractPackageNameFromPackageXML(package_file_path);
-    } else if (boost::filesystem::exists(parent / "manifest.xml")) {
-#if BOOST_FILESYSTEM_VERSION >= 3
-      std::string package = parent.filename().string();
-#else
-      std::string package = parent.filename();
-#endif
-      std::string package_path = ros::package::getPath(package);
-
-      // package_path is a substr of passed plugin xml path
-      if (0 == plugin_xml_file_path.find(package_path)) {
-        package_name = package;
-        break;
-      }
     }
 
-    // Recursive case - hop one folder up
-    parent = parent.parent_path().string();
+    // Recursive case - hop one folder up and store current parent
+    // parent_path() returns the current path if we reached the root.
+    p = parent;
+    parent = parent.parent_path();
 
     // Base case - reached root and cannot find what we're looking for
-    if (parent.string().empty()) {
+    if (parent.string().empty() || (p == parent)) {
       return "";
     }
   }
@@ -559,15 +598,7 @@ template<class T>
 std::string ClassLoader<T>::getPathSeparator()
 /***************************************************************************/
 {
-#if BOOST_FILESYSTEM_VERSION >= 3
-#  ifdef _WIN32
-     return boost::filesystem::path("/").string();
-#  else
-     return boost::filesystem::path("/").native();
-#  endif
-#else
-  return boost::filesystem::path("/").external_file_string();
-#endif
+  return std::string(1, rcpputils::fs::kPreferredSeparator);
 }
 
 
@@ -590,13 +621,6 @@ std::vector<std::string> ClassLoader<T>::getRegisteredLibraries()
   return lowlevel_class_loader_.getRegisteredLibraries();
 }
 
-template<class T>
-std::string ClassLoader<T>::getROSBuildLibraryPath(const std::string & exporting_package_name)
-/***************************************************************************/
-{
-  return ros::package::getPath(exporting_package_name);
-}
-
 template<class T>
 bool ClassLoader<T>::isClassAvailable(const std::string & lookup_name)
 /***************************************************************************/
@@ -608,7 +632,7 @@ template<class T>
 std::string ClassLoader<T>::joinPaths(const std::string & path1, const std::string & path2)
 /***************************************************************************/
 {
-  boost::filesystem::path p1(path1);
+  rcpputils::fs::path p1(path1);
   return (p1 / path2).string();
 }
 
@@ -618,21 +642,13 @@ void ClassLoader<T>::loadLibraryForClass(const std::string & lookup_name)
 {
   ClassMapIterator it = classes_available_.find(lookup_name);
   if (it == classes_available_.end()) {
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Class %s has no mapping in classes_available_.",
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
+      "Class %s has no mapping in classes_available_.",
       lookup_name.c_str());
     throw pluginlib::LibraryLoadException(getErrorStringForUnknownClass(lookup_name));
   }
 
   std::string library_path = getClassLibraryPath(lookup_name);
-  if ("" == library_path) {
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "No path could be found to the library containing %s.",
-      lookup_name.c_str());
-    std::ostringstream error_msg;
-    error_msg << "Could not find library corresponding to plugin " << lookup_name <<
-      ". Make sure the plugin description XML file has the correct name of the "
-      "library and that the library actually exists.";
-    throw pluginlib::LibraryLoadException(error_msg.str());
-  }
 
   try {
     lowlevel_class_loader_.loadLibrary(library_path);
@@ -653,7 +669,7 @@ void ClassLoader<T>::processSingleXMLPluginFile(
   ClassDesc> & classes_available)
 /***************************************************************************/
 {
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Processing xml file %s...", xml_file.c_str());
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader", "Processing xml file %s...", xml_file.c_str());
   tinyxml2::XMLDocument document;
   document.LoadFile(xml_file.c_str());
   tinyxml2::XMLElement * config = document.RootElement();
@@ -687,20 +703,20 @@ void ClassLoader<T>::processSingleXMLPluginFile(
   while (library != NULL) {
     const char* path = library->Attribute("path");
     if (NULL == path) {
-      ROS_ERROR_NAMED("pluginlib.ClassLoader",
+      RCUTILS_LOG_ERROR_NAMED("pluginlib.ClassLoader",
         "Attribute 'path' in 'library' tag is missing in %s.", xml_file.c_str());
       continue;
     }
     std::string library_path(path);
     if (0 == library_path.size()) {
-      ROS_ERROR_NAMED("pluginlib.ClassLoader",
-        "Attribute 'path' in 'library' tag is missing in %s.", xml_file.c_str());
+      RCUTILS_LOG_ERROR_NAMED("pluginlib.ClassLoader",
+        "Failed to find Path Attirbute in library element in %s", xml_file.c_str());
       continue;
     }
 
     std::string package_name = getPackageFromPluginXMLFilePath(xml_file);
     if ("" == package_name) {
-      ROS_ERROR_NAMED("pluginlib.ClassLoader",
+      RCUTILS_LOG_ERROR_NAMED("pluginlib.ClassLoader",
         "Could not find package manifest (neither package.xml or deprecated "
         "manifest.xml) at same directory level as the plugin XML file %s. "
         "Plugins will likely not be exported properly.\n)",
@@ -728,11 +744,11 @@ void ClassLoader<T>::processSingleXMLPluginFile(
       std::string lookup_name;
       if (class_element->Attribute("name") != NULL) {
         lookup_name = class_element->Attribute("name");
-        ROS_DEBUG_NAMED("pluginlib.ClassLoader",
+        RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
           "XML file specifies lookup name (i.e. magic name) = %s.",
           lookup_name.c_str());
       } else {
-        ROS_DEBUG_NAMED("pluginlib.ClassLoader",
+        RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
           "XML file has no lookup name (i.e. magic name) for class %s, "
           "assuming lookup_name == real class name.",
           derived_class.c_str());
@@ -766,7 +782,7 @@ template<class T>
 void ClassLoader<T>::refreshDeclaredClasses()
 /***************************************************************************/
 {
-  ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Refreshing declared classes.");
+  RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader", "Refreshing declared classes.");
   // determine classes not currently loaded for removal
   std::list<std::string> remove_classes;
   for (std::map<std::string, ClassDesc>::const_iterator it = classes_available_.begin();
@@ -785,7 +801,7 @@ void ClassLoader<T>::refreshDeclaredClasses()
   }
 
   // add new classes
-  plugin_xml_paths_ = getPluginXmlPaths(package_, attrib_name_, true);
+  plugin_xml_paths_ = getPluginXmlPaths(package_, attrib_name_);
   std::map<std::string, ClassDesc> updated_classes = determineAvailableClasses(plugin_xml_paths_);
   for (std::map<std::string, ClassDesc>::const_iterator it = updated_classes.begin();
     it != updated_classes.end(); it++)
@@ -816,7 +832,8 @@ int ClassLoader<T>::unloadLibraryForClass(const std::string & lookup_name)
   ClassMapIterator it = classes_available_.find(lookup_name);
   if (it != classes_available_.end() && it->second.resolved_library_path_ != "UNRESOLVED") {
     std::string library_path = it->second.resolved_library_path_;
-    ROS_DEBUG_NAMED("pluginlib.ClassLoader", "Attempting to unload library %s for class %s",
+    RCUTILS_LOG_DEBUG_NAMED("pluginlib.ClassLoader",
+      "Attempting to unload library %s for class %s",
       library_path.c_str(), lookup_name.c_str());
     return unloadClassLibraryInternal(library_path);
   } else {
